structure Interpreter =
struct

open Command;

exception ValueException of string;
exception SyntaxException of string;

fun state( C( p, C( d, C( pp, C( f, ff ) ) ) ) ) =
    let
        val res = ""
        val res = res ^ "p =  " ^ Value.toString( p ) ^ "\n";
        val res = res ^ "d =  " ^ Value.toString( d ) ^ "\n";
        val res = res ^ "pp = " ^ Value.toString( pp ) ^ "\n";
        val res = res ^ "f =  " ^ Value.toString( f ) ^ "\n";
        val res = res ^ "ff = " ^ Value.toString( ff ) ^ "\n";
    in
        res
    end
  | state( v ) = ""
;

fun applyHd( N ) = 
    raise ValueException( "cannot take head of nil" )
  | applyHd( C( v1, v2 ) ) = 
    v1
;

fun applyTl( N ) = 
    raise ValueException( "cannot take tail of nil" )
  | applyTl( C( v1, v2 ) ) = 
    v2
;

fun evaluate( VAR, var, time ) = 
    ( var, time + 1 )
  | evaluate( VAL v, var, time ) = 
    ( v, time + 1 )
  | evaluate( CONS( e1, e2 ), var, time ) =
    let
        val ( v1, t1 ) = evaluate( e1, var, 0 );
        val ( v2, t2 ) = evaluate( e2, var, 0 );
    in
        ( C( v1, v2 ), time + t1 + t2 + 1 )
    end
  | evaluate( HD( e ), var, time ) =
    let
        val ( v, t ) = evaluate( e, var, 0 );
    in
        ( applyHd( v ), time + t + 1 )
    end
  | evaluate( TL( e ), var, time ) =
    let
        val ( v, t ) = evaluate( e, var, 0 );
    in
        ( applyTl( v ), time + t + 1 )
    end
  | evaluate( EQ( e1, e2 ), var, time ) =
    let
        val( v1, t1 ) = evaluate( e1, var, 0 );
        val( v2, t2 ) = evaluate( e2, var, 0 );
    in
        if v1 = v2
        then ( C(N,N), time + t1 + t2 + 1 )
        else ( N, time + t1 + t2 + 1 )
    end      
;

fun interpret( ASSIGN( e ), var, ls, time ) =
    let
        val ( new_var, time1 ) = evaluate( e, var, 0 );
    in
        ( new_var, time + time1 + 1 )
    end
  | interpret( IF2( e, c1, c2 ), var, ls, time ) =
    let
        val( v, t ) = evaluate( e, var, 0 );
    in
        if v <> N
        then interpret( c1, var, ls, time + t + 1 )
        else interpret( c2, var, ls, time + t + 1 )
    end
  | interpret( IF1( e, c ), var, ls, time ) =
    let
        val( v, t ) = evaluate( e, var, 0 );
    in
        if v <> N
        then interpret( c, var, ls, time + t + 1 )
        else ( var, time + t + 1 )
    end
  | interpret( SEQ( REPEAT, c2 ), var, ls, time ) =
    interpret( REPEAT, var, ls, time + 1 )
  | interpret( SEQ( c1, c2 ), var, ls, time ) =
    let
        val ( var1, time1 ) = interpret( c1, var, ls, 0 );
        val ( var2, time2 ) = interpret( c2, var1, ls, 0 );
    in
        ( var2, time + time1 + time2 + 1 )
    end
  | interpret( LOOP( c ), var, ls, time ) =
    let
(*
        val _ = print( "loop\n" );
        val _ = print( state( var ) ^ "\n" );
*)
    in
        interpret( c, var, c :: ls, time + 1 )
    end
  | interpret( REPEAT, var, [], time ) =
    raise SyntaxException( "repeat is missing a loop" )
  | interpret( REPEAT, var, c :: ls, time ) =
    let
(*
        val _ = print( "repeat\n" );
        val _ = print( state( var ) ^ "\n" );
*)
    in
        interpret( c, var, c :: ls, time + 1 )
    end
;

fun timed_interpretation( prog, init ) =
    let
        val ( var, time ) = interpret( prog, init, [], 0 );
    in
        ( var, time )
    end
;

end
;
