structure Normalization =
struct

open Command;
open Interpreter;

exception NormalizationException of string;

fun hasRepeats( ASSIGN( e ) ) =
    false
  | hasRepeats( SEQ( c1, c2 ) ) =
    hasRepeats( c1 ) orelse hasRepeats( c2 )
  | hasRepeats( LOOP( c ) ) =
    false
  | hasRepeats( IF1( e, c ) ) =
    hasRepeats( c )
  | hasRepeats( IF2( e, c1, c2 ) ) =
    hasRepeats( c1 ) orelse hasRepeats( c2 )
  | hasRepeats( REPEAT ) =
    true
;

fun normalize( ASSIGN( e ) ) =
    ASSIGN( e )
    
  | normalize( SEQ( ASSIGN( e1 ), ASSIGN( e2 ) ) ) =
    ASSIGN( compact( substitute( e1, e2 ) ) )

  | normalize( SEQ( ASSIGN( e1 ), SEQ( ASSIGN( e2 ), c3 ) ) ) =
    SEQ( ASSIGN( compact( substitute( e1, e2 ) ) ), c3 )

  | normalize( IF1( e, c ) ) =
    IF2( e, c, ASSIGN( VAR ) )

  | normalize( SEQ( ASSIGN( e1 ), IF2( e2, c1, c2 ) ) ) =
    IF2( compact( substitute( e1, e2 ) ), 
         SEQ( ASSIGN( e1 ), c1 ), 
         SEQ( ASSIGN( e1 ), c2 ) )

  | normalize( SEQ( IF2( e, c1, c2 ), c3 ) ) =
    IF2( e, SEQ( c1, c3 ), SEQ( c2, c3 ) )

  | normalize( IF2( VAL N, c1, c2 ) ) =
    c2

  | normalize( IF2( VAL v, c1, c2 ) ) =
    c1

  | normalize( LOOP( c ) ) =
    if not( hasRepeats( c ) )
    then c
    else LOOP( normalize( c ) )

  | normalize( SEQ( c1, ASSIGN( VAL v ) ) ) =
    ASSIGN( VAL v )

  | normalize( SEQ( ASSIGN( VAL v ), c2 ) ) =
    ASSIGN( VAL ( #1( interpret( c2, v, [], 0 ) ) ) )

  | normalize( SEQ( REPEAT, c ) ) = 
    REPEAT

  | normalize( SEQ( LOOP( c1 ), SEQ( c2, c3 ) ) ) =
    if not( hasRepeats( c2 ) )
    then SEQ( LOOP( SEQ( c1, c2 ) ), c3 )
    else SEQ( LOOP( c1 ), normalize( SEQ( c2, c3 ) ) )

  | normalize( SEQ( LOOP( c1 ), c2 ) ) =
    if not( hasRepeats( c2 ) )
    then LOOP( SEQ( c1, c2 ) )
    else SEQ( normalize( LOOP( c1 ) ), normalize( c2 ) )

  | normalize( SEQ( ASSIGN( VAR ), c2 ) ) =
    c2

  | normalize( SEQ( c1, ASSIGN( VAR ) ) ) =
    c1

  | normalize( SEQ( SEQ( c1, c2 ), c3 ) ) =
    normalize( SEQ( c1, SEQ( c2, c3 ) ) )

  | normalize( SEQ( c1, c2 ) ) =
    let
        val c1n = normalize( c1 );
    in
        if c1n = c1
        then SEQ( c1, normalize( c2 ) )
        else SEQ( c1n, c2 )
    end
  | normalize( IF2( e, c1, c2 ) ) =
    let
        val c1n = normalize( c1 );
    in
        if c1n = c1
        then IF2( e, c1, normalize( c2 ) )
        else IF2( e, c1n, c2 )
    end
  | normalize( REPEAT ) =
    REPEAT
;

end;
