structure Pruner =
struct

open Command;
open Expression;

exception PruningError of string;

fun isPrefix( e1, VAR ) = 
    e1 = VAR
  | isPrefix( e1, VAL v ) = 
    false
  | isPrefix( e1, HD( e ) ) = 
    if e1 = e
    then true
    else isPrefix( e1, e )
  | isPrefix( e1, TL( e ) ) = 
    if e1 = e
    then true
    else isPrefix( e1, e )
  | isPrefix( e1, e ) =
    raise PruningError( "Prefix is not defined for CONS nor EQ" )
;

fun eq( VAR, VAR ) = true
  | eq( HD( e1 ), HD( e2 ) ) =
    eq( e1, e2 )
  | eq( TL( e1 ), TL( e2 ) ) =
    eq( e1, e2 )
  | eq( e1, e2 ) = false
;

fun leq( VAR, VAR ) = true
  | leq( VAR, HD( e ) ) = true
  | leq( VAR, TL( e ) ) = true
  | leq( HD( e1 ), HD( e2 ) ) =
    leq( e1, e2 )
  | leq( HD( e1 ), TL( e2 ) ) =
    if( e1 = e2 )
    then true
    else leq( e1, e2 )
  | leq( TL( e1 ), TL( e2 ) ) =
    leq( e1, e2 )
  | leq( TL( e1 ), HD( e2 ) ) =
    if( e1 = e2 )
    then false
    else leq( e1, e2 )
  | leq( e1, e2 ) =
    raise PruningError( "Cannot compare " ^ 
                        toString( e1 ) ^ 
                        " and " ^ 
                        toString( e2 ) )
;

fun sort_used( [] ) = []
  | sort_used( [ v ] ) = [ v ]
  | sort_used( v1 :: v2 :: vs ) =
    if size( v1 ) < size( v2 )
    then v1 :: sort_used( v2 :: vs )
    else if size( v1 ) > size( v2 )
    then v2 :: sort_used( v1 :: vs )
    else if leq( v1, v2 )
    then v1 :: sort_used( v2 :: vs )
    else v2 :: sort_used( v1 :: vs )
;

fun merge( [] ) = []
  | merge( v :: [] ) = v :: []
  | merge( HD( e1 ) :: TL( e2 ) :: vs ) =
    if( e1 = e2 )
    then merge( e1 :: vs )
    else HD( e1 ) :: merge( TL( e2 ) :: vs )
  | merge( v1 :: v2 :: vs ) = v1 :: v2 :: vs
;

fun update_found_list( e, res ) =
    let
        val found = 
            NONE <> List.find( fn x => x = e ) res;
        val containsPrefix = 
            NONE <> List.find( fn x => isPrefix( x, e ) ) res;
        val l1 = 
            List.filter ( fn x => not ( isPrefix( e, x ) ) ) res;
        val result = 
            if found
            then res
            else if containsPrefix
            then res
            else sort_used( e :: l1 )
        val result = merge( result );
    in
        result
    end
;

fun find_used_exp( VAL v, res ) = 
    res
  | find_used_exp( CONS( e1, e2 ), res ) =
    find_used_exp( e2, find_used_exp( e1, res ) )
  | find_used_exp( EQ( e1, e2 ), res ) = 
    find_used_exp( e2, find_used_exp( e1, res ) )
  | find_used_exp( e, res ) =
    update_found_list( e, res )
;

fun find_used_cmd( ASSIGN( e ), res, repeat ) = 
    find_used_exp( e, res )
  | find_used_cmd( SEQ( c1, c2 ), res, repeat ) =
    find_used_cmd( c2,
                   find_used_cmd( c1, res, repeat ),
                   repeat )
  | find_used_cmd( LOOP( c ), res, repeat ) =
    find_used_cmd( c, res, [] )
  | find_used_cmd( IF2( e, c1, c2 ), res, repeat ) =
    find_used_cmd( c2,
                   find_used_cmd( c1,
                                  find_used_exp( e, res ),
                                  repeat ),
                   repeat )
  | find_used_cmd( IF1( e, c ), res, repeat ) =
    find_used_cmd( c, find_used_exp( e, res ), repeat )
  | find_used_cmd( REPEAT, res, [] ) = res
  | find_used_cmd( REPEAT, res, e :: es ) =
    find_used_cmd( REPEAT, update_found_list( e, res ), es )
;

fun construct_new_e( e, [], path ) =
    raise PruningError( "construct_new_e needs usage " ^
                        "information to continue" )
  | construct_new_e( e, ev :: [], path ) =
    ( compact( substitute( e, ev ) ), [ ( ev, path ) ] )
  | construct_new_e( e, ev :: evs, path ) =
    let
        val new_e = compact( substitute( e, ev ) );
        val ( rest, rest_map ) = 
            construct_new_e( e, evs, TL( path ) );
    in
        ( CONS( new_e, rest ), ( ev, HD( path ) ) :: rest_map )
    end

fun apply_mapping( VAR, mapping ) =
    let
        val found = List.find (fn( x, y ) => x = VAR ) mapping;
    in
        if found <> NONE
        then #2( valOf( found ) )
        else VAR
    end
  | apply_mapping( VAL v, mapping ) =
    VAL v
  | apply_mapping( HD( e ), mapping ) =
    let
        val found = List.find( fn( x, y ) => x = HD( e ) ) mapping;
    in
        if found <> NONE
        then #2( valOf( found ) )
        else HD( apply_mapping( e, mapping ) )
    end
  | apply_mapping( TL( e ), mapping ) =
    let
        val found = List.find( fn( x, y ) => x = TL( e ) ) mapping;
    in
        if found <> NONE
        then #2( valOf( found ) )
        else TL( apply_mapping( e, mapping ) )
    end
  | apply_mapping( CONS( e1, e2 ), mapping ) =
    CONS( apply_mapping( e1, mapping ), 
          apply_mapping( e2, mapping ) )
  | apply_mapping( EQ( e1, e2 ), mapping ) =
    EQ( apply_mapping( e1, mapping ), 
        apply_mapping( e2, mapping ) )
;

fun remove_unused( ASSIGN( e ), cmap, used_repeat, [] ) =
    let
        val new_e = apply_mapping( e, cmap )
    in
        ( ASSIGN( new_e ), cmap )
    end
  | remove_unused( ASSIGN( e ), cmap, used_repeat, used_escape ) =
    let
        val new_e = apply_mapping( e, cmap )
        val( new_e, mapping ) = 
           construct_new_e( new_e, used_escape, VAR );
    in
        ( ASSIGN( new_e ), mapping )
    end

  | remove_unused( SEQ( c1, c2 ), cmap, used_repeat, used_escape ) =
    let
        val used_c2 = find_used_cmd( c2, [], used_repeat );
        val( new_c1, cmap_1 ) = 
           remove_unused( c1, cmap, used_repeat, used_c2 );
        val( new_c2, cmap_2 ) = 
           remove_unused( c2, cmap_1, used_repeat, used_escape );
    in
        ( SEQ( new_c1, new_c2 ), cmap_2 )
    end
  | remove_unused( LOOP( c ), cmap, used_repeat, used_escape ) =
    let
        val used = find_used_cmd( c, [], [] );
    in
        ( LOOP( #1( remove_unused( c, cmap, 
                                   used, used_escape ) ) ), 
          [] )
    end
  | remove_unused( IF1( e, c ), cmap, used_repeat, used_escape ) =
    raise PruningError( "Normalization should make sure " ^ 
                        "that if1's are converted to if2's" )
  | remove_unused( IF2( e, c1, c2 ), cmap, 
                   used_repeat, used_escape ) =
    ( IF2( apply_mapping( e, cmap ), 
           #1( remove_unused( c1, cmap, 
                              used_repeat, used_escape ) ), 
           #1( remove_unused( c2, cmap, 
                              used_repeat, used_escape ) ) ), [] )
  | remove_unused( REPEAT, cmap, used_repeat, used_escape ) =
    ( REPEAT, [] )
;

end
;
