structure NormalizationA =
struct

open CommandA;

exception NormalizationException of string;

fun hasRepeats( ASSIGN( e ) ) =
    false
  | hasRepeats( ASSIGNS( e ) ) =
    false
  | hasRepeats( SEQ( c1, c2 ) ) =
    hasRepeats( c1 ) orelse hasRepeats( c2 )
  | hasRepeats( LOOP( c ) ) =
    false
  | hasRepeats( IF2S( e, c1, c2 ) ) =
    hasRepeats( c1 ) orelse hasRepeats( c2 )
  | hasRepeats( IF2D( e, c1, c2 ) ) =
    hasRepeats( c1 ) orelse hasRepeats( c2 )
  | hasRepeats( REPEAT ) =
    true
  | hasRepeats( UNROLL ) =
    true
  | hasRepeats( UNROLL_LG ) =
    true
  | hasRepeats( ESCAPE ) =
    true
;

fun normalize( ASSIGN( e ) ) =
    ASSIGN( e )
  | normalize( ASSIGNS( e ) ) =
    ASSIGNS( e )

  | normalize( SEQ( ASSIGN( e1 ), ASSIGN( e2 ) ) ) =
    ASSIGN( compact( substitute( e1, e2 ) ) )

  | normalize( SEQ( ASSIGNS( e1 ), ASSIGNS( e2 ) ) ) =
    ASSIGNS( compact( substitute( e1, e2 ) ) )

  | normalize( SEQ( ASSIGN( e1 ), SEQ( ASSIGN( e2 ), c3 ) ) ) =
    SEQ( ASSIGN( compact( substitute( e1, e2 ) ) ), c3 )

  | normalize( SEQ( ASSIGNS( e1 ), SEQ( ASSIGNS( e2 ), c3 ) ) ) =
    SEQ( ASSIGNS( compact( substitute( e1, e2 ) ) ), c3 )

  | normalize( SEQ( ASSIGN( e1 ), IF2S( e2, c1, c2 ) ) ) =
    IF2S( compact( substitute( e1, e2 ) ), 
          SEQ( ASSIGN( e1 ), c1 ), 
          SEQ( ASSIGN( e1 ), c2 ) )

  | normalize( SEQ( ASSIGN( e1 ), IF2D( e2, c1, c2 ) ) ) =
    IF2D( compact( substitute( e1, e2 ) ), 
          SEQ( ASSIGN( e1 ), c1 ), 
          SEQ( ASSIGN( e1 ), c2 ) )

  | normalize( SEQ( ASSIGNS( e1 ), IF2S( e2, c1, c2 ) ) ) =
    IF2S( compact( substitute( e1, e2 ) ), 
          SEQ( ASSIGNS( e1 ), c1 ), 
          SEQ( ASSIGNS( e1 ), c2 ) )

  | normalize( SEQ( ASSIGNS( e1 ), IF2D( e2, c1, c2 ) ) ) =
    IF2D( compact( substitute( e1, e2 ) ), 
          SEQ( ASSIGNS( e1 ), c1 ), 
          SEQ( ASSIGNS( e1 ), c2 ) )

  | normalize( SEQ( IF2S( e, c1, c2 ), c3 ) ) =
    IF2S( e, SEQ( c1, c3 ), SEQ( c2, c3 ) )
  | normalize( SEQ( IF2D( e, c1, c2 ), c3 ) ) =
    IF2D( e, SEQ( c1, c3 ), SEQ( c2, c3 ) )

  | normalize( IF2S( VALA N, c1, c2 ) ) =
    c2
  | normalize( IF2D( VALA N, c1, c2 ) ) =
    c2

  | normalize( IF2S( VALA v, c1, c2 ) ) =
    c1
  | normalize( IF2D( VALA v, c1, c2 ) ) =
    c1

  | normalize( LOOP( c ) ) =
    if not( hasRepeats( c ) )
    then c
    else LOOP( normalize( c ) )

  | normalize( SEQ( c1, ASSIGN( VALA v ) ) ) =
    ASSIGN( VALA v )
  | normalize( SEQ( c1, ASSIGNS( VALA v ) ) ) =
    ASSIGNS( VALA v )

  | normalize( SEQ( ASSIGN( VALA v ), c2 ) ) =
    ASSIGN( VALA ( #1( Interpreter.interpret( CommandA.erase( c2 ), 
                                              v, [], 0 ) ) ) )
  | normalize( SEQ( ASSIGNS( VALA v ), c2 ) ) =
    ASSIGNS( VALA ( #1( Interpreter.interpret( CommandA.erase( c2 ), 
                                               v, [], 0 ) ) ) )

  | normalize( SEQ( REPEAT, c ) ) = 
    REPEAT

  | normalize( SEQ( UNROLL, c ) ) = 
    UNROLL

  | normalize( SEQ( ESCAPE, c ) ) = 
    ESCAPE

  | normalize( SEQ( UNROLL_LG, c ) ) = 
    UNROLL_LG

  | normalize( SEQ( LOOP( c1 ), SEQ( c2, c3 ) ) ) =
    if not( hasRepeats( c2 ) )
    then SEQ( LOOP( SEQ( c1, c2 ) ), c3 )
    else SEQ( LOOP( c1 ), normalize( SEQ( c2, c3 ) ) )
  | normalize( SEQ( LOOP( c1 ), c2 ) ) =
    if not( hasRepeats( c2 ) )
    then LOOP( SEQ( c1, c2 ) )
    else SEQ( normalize( LOOP( c1 ) ), normalize( c2 ) )

  | normalize( SEQ( ASSIGN( VARA ), c2 ) ) =
    c2
  | normalize( SEQ( ASSIGNS( VARA ), c2 ) ) =
    c2

  | normalize( SEQ( c1, ASSIGN( VARA ) ) ) =
    c1
  | normalize( SEQ( c1, ASSIGNS( VARA ) ) ) =
    c1

  | normalize( SEQ( SEQ( c1, c2 ), c3 ) ) =
    normalize( SEQ( c1, SEQ( c2, c3 ) ) )

  | normalize( SEQ( c1, c2 ) ) =
    let
        val c1n = normalize( c1 );
    in
        if c1n = c1
        then SEQ( c1, normalize( c2 ) )
        else SEQ( c1n, c2 )
    end

  | normalize( IF2S( e, c1, c2 ) ) =
    let
        val c1n = normalize( c1 )
    in
        if c1n = c1
        then IF2S( e, c1, normalize( c2 ) )
        else IF2S( e, c1n, c2 )
    end

  | normalize( IF2D( e, c1, c2 ) ) =
    let
        val c1n = normalize( c1 );
    in
        if c1n = c1
        then IF2D( e, c1, normalize( c2 ) )
        else IF2D( e, c1n, c2 )
    end

  | normalize( REPEAT ) =
    REPEAT
  | normalize( UNROLL ) =
    UNROLL
  | normalize( UNROLL_LG ) =
    UNROLL_LG
  | normalize( ESCAPE ) =
    ESCAPE
;

end;
