structure LPparser =
struct

structure LPLrVals =
  LPLrValsFun(structure Token = LrParser.Token)
structure LPLex = 
  LPLexFun(structure Tokens = LPLrVals.Tokens)
structure LPParser=
  Join( structure ParserData = LPLrVals.ParserData
        structure Lex=LPLex
        structure LrParser=LrParser)

fun invoke lexstream =
    let
        fun print_error (s,i:int,_) =
            TextIO.output(TextIO.stdOut,
                          "Error, line " ^ (Int.toString i) ^ ", " ^ s ^ "\n")
        val result = LPParser.parse( 0, lexstream, print_error, () );
     in
        result
    end

fun parseFile( file ) = 
    let
        val instream = TextIO.openIn( file );
        val lexer = LPParser.makeLexer( fn _ => ( case TextIO.inputLine instream of SOME( s ) => s | NONE => "" ) )
        val dummyEOF = LPLrVals.Tokens.EOF(0,0)
        fun loop lexer =
            let 
                val (result,lexer) = invoke lexer
                val (nextToken,lexer) = LPParser.Stream.get lexer
             in 
                if LPParser.sameToken(nextToken,dummyEOF) 
                then result
                else loop lexer
            end
     in
        loop lexer
    end
;

fun parse() = 
    let
        val lexer = LPParser.makeLexer( fn _ => ( case TextIO.inputLine TextIO.stdIn of SOME( s ) => s | NONE => "" ) )
        val dummyEOF = LPLrVals.Tokens.EOF(0,0)
        fun loop lexer =
            let 
                val (result,lexer) = invoke lexer
                val (nextToken,lexer) = LPParser.Stream.get lexer
             in 
                if LPParser.sameToken(nextToken,dummyEOF) 
                then result
                else loop lexer
            end
     in
        loop lexer
    end
;

end
;
