structure Command =
struct

open Expression;
open Tools;

datatype Command =
         SEQ of Command * Command
       | LOOP of Command
       | ASSIGN of Expression
       | IF2 of Expression * Command * Command
       | IF1 of Expression * Command
       | REPEAT
;

fun toString( SEQ( c1, c2 ) ) = 
    toString( c1 ) ^ ";\n" ^ 
    indentToString( !indent ) ^ toString( c2 )
  | toString( ASSIGN( e ) ) = 
    "X := " ^ Expression.toString( e )
  | toString( LOOP( c ) ) = 
    "loop\n" ^ indentToString( !indent ) ^ "{\n" ^ 
    startBlock() ^ indentToString( !indent ) ^ 
    toString( c ) ^ "\n" ^ 
    endBlock() ^ indentToString( !indent ) ^ "}"
  | toString( IF2( e, c1, c2 ) ) = 
    "if2( " ^ Expression.toString( e ) ^ " )\n" ^ 
    indentToString( !indent ) ^
    "then\n" ^ indentToString( !indent ) ^ "{\n" ^ 
    startBlock() ^ indentToString( !indent ) ^ 
    toString( c1 ) ^ "\n" ^ 
    endBlock() ^ indentToString( !indent ) ^ "}\n" ^
    indentToString( !indent ) ^ "else\n" ^ 
    indentToString( !indent ) ^ "{\n" ^ 
    startBlock() ^ indentToString( !indent ) ^ 
    toString( c2 ) ^ "\n" ^ 
    endBlock() ^ indentToString( !indent ) ^ "}"
  | toString( IF1( e, c ) ) = 
    "if1( " ^ Expression.toString( e ) ^ " )\n" ^ 
    indentToString( !indent ) ^ 
    "then\n" ^ indentToString( !indent ) ^ "{\n" ^ 
    startBlock() ^ indentToString( !indent ) ^ 
    toString( c ) ^ "\n" ^
    endBlock() ^ indentToString( !indent ) ^ "}"
  | toString( REPEAT ) = 
    "repeat"
;

fun newline() = "\n" ^ indentToString( !indent );

fun toStringPP( SEQ( c1, c2 ), max, n ) = 
    toStringPP( c1, max, n ) ^ ";" ^ newline() ^
    toStringPP( c2, max, n )
  | toStringPP( ASSIGN( e ), max, n ) = 
    "X := " ^ 
    Expression.toStringPP( e, max, !indent + 5 )
  | toStringPP( LOOP( c ), max, n ) = 
    "loop{" ^ startBlock() ^ newline() ^
    toStringPP( c, max, n ) ^ endBlock() ^ newline() ^ "}"
  | toStringPP( IF2( e, c1, c2 ), max, n ) = 
    "if2( " ^ Expression.toStringPP( e, max, !indent + 4 ) ^
    " ) {" ^ startBlock() ^ newline() ^
    toStringPP( c1, max, n ) ^ endBlock() ^ newline() ^
    "}{" ^ startBlock() ^ newline() ^
    toStringPP( c2, max, n ) ^ endBlock() ^ newline() ^ "}"
  | toStringPP( IF1( e, c ), max, n ) = 
    "if1( " ^ Expression.toStringPP( e, max, !indent + 4 ) ^
    " ) {" ^ startBlock() ^ newline() ^
    toStringPP( c, max, n ) ^ endBlock() ^ newline() ^ "}"
  | toStringPP( REPEAT, max, n ) = "repeat"
;

fun size( ASSIGN( e ) ) = Expression.size( e )
  | size( SEQ( c1, c2 ) ) =
    size( c1 ) + size( c2 ) + 1
  | size( LOOP( c ) ) =
    size( c ) + 1
  | size( IF2( e, c1, c2 ) ) =
    Expression.size( e ) + size( c1 ) + size( c2 ) + 1
  | size( IF1( e, c ) ) =
    Expression.size( e ) + size( c )
  | size( REPEAT ) = 1

end
;
