structure CommandA =
struct

open Tools;
open ExpressionA;

datatype Command =
         ASSIGN of ExpressionA.Expression
       | ASSIGNS of ExpressionA.Expression
       | SEQ of Command * Command
       | LOOP of Command
       | IF2S of ExpressionA.Expression * Command * Command
       | IF2D of ExpressionA.Expression * Command * Command
       | REPEAT
       | UNROLL
       | ESCAPE
       | UNROLL_LG
;

fun toString( ASSIGN( e ) ) = 
    "X := " ^ ExpressionA.toString( e )
  | toString( ASSIGNS( e ) ) = 
    "X :=s " ^ ExpressionA.toString( e )
  | toString( SEQ( c1, c2 ) ) = 
    toString( c1 ) ^ ";\n" ^ indentToString( !indent ) ^ 
    toString( c2 )
  | toString( LOOP( c ) ) = 
    "loop{\n" ^ startBlock() ^ indentToString( !indent ) ^ 
    toString( c ) ^ "\n" ^ 
    endBlock() ^ indentToString( !indent ) ^ "}"
  | toString( IF2S( e, c1, c2 ) ) =
    "if2s( " ^ ExpressionA.toString( e ) ^ ")\n" ^ 
    indentToString( !indent ) ^
    "{\n" ^ startBlock() ^ indentToString( !indent ) ^ 
    toString( c1 ) ^ "\n" ^ 
    endBlock() ^ indentToString( !indent ) ^ "},\n" ^
    indentToString( !indent ) ^ "{\n" ^ 
    startBlock() ^ indentToString( !indent ) ^ toString( c2 ) ^ 
    "\n" ^ 
    endBlock() ^ indentToString( !indent ) ^ "}"
  | toString( IF2D( e, c1, c2 ) ) =
    "if2d( " ^ ExpressionA.toString( e ) ^ ")\n" ^ 
    indentToString( !indent ) ^
    "{\n" ^ startBlock() ^ indentToString( !indent ) ^ 
    toString( c1 ) ^ "\n" ^ 
    endBlock() ^ indentToString( !indent ) ^ "},\n" ^
    indentToString( !indent ) ^ "{\n" ^ 
    startBlock() ^ indentToString( !indent ) ^ toString( c2 ) ^ 
    "\n" ^ 
    endBlock() ^ indentToString( !indent ) ^ "}"
  | toString( REPEAT ) = 
    "repeat"
  | toString( UNROLL ) = 
    "unroll"
  | toString( ESCAPE ) = 
    "unroll_up"
  | toString( UNROLL_LG ) = 
    "unroll_lg"
;

fun erase( ASSIGN( e ) ) = Command.ASSIGN( ExpressionA.erase( e ) )
  | erase( ASSIGNS( e ) ) = Command.ASSIGN( ExpressionA.erase( e ) )
  | erase( SEQ( c1, c2 ) ) = Command.SEQ( erase( c1 ), erase( c2 ) )
  | erase( LOOP( c ) ) = Command.LOOP( erase( c ) )
  | erase( IF2S( e, c1, c2 ) ) = 
    Command.IF2( ExpressionA.erase( e ), erase( c1 ), erase( c2 ) )
  | erase( IF2D( e, c1, c2 ) ) =
    Command.IF2( ExpressionA.erase( e ), erase( c1 ), erase( c2 ) )
  | erase( REPEAT ) = Command.REPEAT
  | erase( UNROLL ) = Command.REPEAT
  | erase( ESCAPE ) = Command.REPEAT
  | erase( UNROLL_LG ) = Command.REPEAT
;


end
;
