structure Expression =
struct

open Value;

exception ExpressionError of string;

datatype Expression =
         VAR
       | VAL of Value
       | HD of Expression
       | TL of Expression
       | CONS of Expression * Expression
       | EQ of Expression * Expression
;

fun toString( VAR ) = 
    "X"
  | toString( VAL v ) = 
    Value.toString( v )
  | toString( CONS( e1, e2 ) ) = 
    "cons( " ^ toString( e1 ) ^ ", " ^ toString( e2 ) ^ " )"
  | toString( HD( e ) ) = 
    "hd( " ^ toString( e ) ^ " )"
  | toString( TL( e ) ) = 
    "tl( " ^ toString( e ) ^ " )"
  | toString( EQ( e1, e2 ) ) = 
    "=?( " ^ toString( e1 ) ^ ", " ^ toString( e2 ) ^ " )"
;

fun toStringPP( VAR, max, n ) = "X"
  | toStringPP( VAL v, max, n ) = Value.toStringPP( v, max, n )
  | toStringPP( CONS( e1, e2 ), max, n ) =
    let
        val e1s = toString( e1 );
        val e2s = toString( e2 );
        val firstSize = n + 7 + String.size( e1s );
        val totalSize = n + 9 +
                        String.size( e1s ) +
                        String.size( e2s );
    in
        if totalSize < max
        then "cons( " ^ e1s ^ ", " ^ e2s ^ " )"
        else "cons(\n" ^ 
             indentToString( n + 2 ) ^
             toStringPP( e1, max, n + 2 ) ^ ",\n" ^ 
             indentToString( n + 2 ) ^ 
             toStringPP( e2, max, n + 2 ) ^ " )"
    end
  | toStringPP( HD( e ), max, n ) = 
    "hd( " ^ toStringPP( e, max, n + 4 ) ^ " )"
  | toStringPP( TL( e ), max, n ) = 
    "tl( " ^ toStringPP( e, max, n + 4 ) ^ " )"
  | toStringPP( EQ( e1, e2 ), max, n ) =
    let
        val e1s = toString( e1 );
        val e2s = toString( e2 );
        val firstSize = n + 5 + String.size( e1s );
        val totalSize = n + 7 +
                        String.size( e1s ) +
                        String.size( e2s );
    in
        if totalSize < max
        then "=?( " ^ e1s ^ ", " ^ e2s ^ " )"
        else "=?(\n" ^ 
             indentToString( n + 2 ) ^
             toStringPP( e1, max, n + 2 ) ^ ",\n" ^ 
             indentToString( n + 2 ) ^ 
             toStringPP( e2, max, n + 2 ) ^ "\n" ^
             indentToString( n ) ^ " )"
    end
;

fun size( VAR ) = 1
  | size( VAL v ) = 1
  | size( HD( e ) ) = 1 + size( e )
  | size( TL( e ) ) = 1 + size( e )
  | size( CONS( e1, e2 ) ) = 1 + size( e1 ) + size( e2 )
  | size( EQ( e1, e2 ) ) = 1 + size( e1 ) + size( e2 )
;

fun compact( EQ( VAL v1, VAL v2 ) ) =
    if( v1 = v2 )
    then VAL( C(N,N) )
    else VAL N
  | compact( EQ( e1, e2 ) ) =
    EQ( e1, e2 )
  | compact( CONS( VAL v1, VAL v2 ) ) = 
    VAL( C( v1, v2 ) )
  | compact( CONS( e1, e2 ) ) = 
    CONS( compact( e1 ), compact( e2 ) )
  | compact( HD( e ) ) = 
    compact_hd( compact( e ) )
  | compact( TL( e ) ) = 
    compact_tl( compact( e ) )
  | compact( e ) = e

and compact_tl( VAL( C( v1, v2 ) ) ) = VAL v2
  | compact_tl( CONS( e1, e2 ) ) = compact( e2 )
  | compact_tl( e ) = TL( e )

and compact_hd( VAL( C( v1, v2 ) ) ) = VAL v1
  | compact_hd( CONS( e1, e2 ) ) = compact( e1 )
  | compact_hd( e ) = HD( e )
;

fun substitute( e, VAR ) = e
  | substitute( e, VAL v ) = VAL v
  | substitute( e, CONS( e1, e2 ) ) = 
    CONS( substitute( e, e1 ), substitute( e, e2 ) )
  | substitute( e, HD( e1 ) ) = 
    HD( substitute( e, e1 ) )
  | substitute( e, TL( e1 ) ) = 
    TL( substitute( e, e1 ) )
  | substitute( e, EQ( e1, e2 ) ) =
    EQ( compact( substitute( e, e1 ) ), 
        compact( substitute( e, e2 ) ) )
;

fun isValue( VAR ) = false
  | isValue( VAL v ) = true
  | isValue( e ) = false

fun value( VAL v ) = v
  | value( e ) = raise ExpressionError( "E is not a value" )
;

fun isCons( CONS( e1, e2 ) ) = true
  | isCons( e ) = false

end
;
