structure ExpressionA =
struct

open Value;

datatype Expression =
         VARA
       | VALA of Value.Value
       | HDA of Expression
       | TLA of Expression
       | CONSA of Expression * Expression
       | EQA of Expression * Expression
       | LIFT of Expression
       | STATIC of Expression
;

fun toString( VARA ) = 
    "X"
  | toString( VALA v ) = 
    Value.toString( v )
  | toString( CONSA( e1, e2 ) ) = 
    "cons( " ^ toString( e1 ) ^ ", " ^ toString( e2 ) ^ " )"
  | toString( HDA( e ) ) = 
    "hd( " ^ toString( e ) ^ " )"
  | toString( TLA( e ) ) = 
    "tl( " ^ toString( e ) ^ " )"
  | toString( EQA( e1, e2 ) ) = 
    "=?( " ^ toString( e1 ) ^ ", " ^ toString( e2 ) ^ " )"
  | toString( LIFT( e ) ) = 
    "lift( " ^ toString( e ) ^ " )"
  | toString( STATIC( e ) ) = 
    "static( " ^ toString( e ) ^ " )"
; 

fun erase( VARA ) = 
    Expression.VAR
  | erase( VALA v ) = 
    Expression.VAL v
  | erase( CONSA( e1, e2 ) ) = 
    Expression.CONS( erase( e1 ), erase( e2 ) )
  | erase( HDA( e ) ) = 
    Expression.HD( erase( e ) )
  | erase( TLA( e ) ) = 
    Expression.TL( erase( e ) )
  | erase( EQA( e1, e2 ) ) = 
    Expression.EQ( erase( e1 ), erase( e2 ) )
  | erase( LIFT( e ) ) = 
    erase( e )
  | erase( STATIC( e ) ) = 
    erase( e )
; 

fun compact( EQA( VALA v1, VALA v2 ) ) =
    if( v1 = v2 )
    then VALA( C(N,N) )
    else VALA N
  | compact( EQA( e1, e2 ) ) =
    EQA( e1, e2 )
  | compact( CONSA( VALA v1, VALA v2 ) ) = 
    VALA( C( v1, v2 ) )
  | compact( CONSA( e1, e2 ) ) = 
    compact_static( CONSA( compact( e1 ), compact( e2 ) ) )
  | compact( HDA( e ) ) = 
    compact_hd( compact( e ) )
  | compact( TLA( e ) ) = 
    compact_tl( compact( e ) )
  | compact( LIFT( e ) ) = 
    LIFT( compact( e ) )
  | compact( STATIC( e ) ) = 
    compact_static( STATIC( compact e ) )
  | compact( e ) = 
    e

and compact_tl( VALA( C( v1, v2 ) ) ) = 
    VALA v2
  | compact_tl( CONSA( e1, e2 ) ) = 
    compact( e2 )
  | compact_tl( STATIC( e ) ) = 
    compact_static( STATIC( compact_tl( e ) ) )
  | compact_tl( e ) = 
    TLA( e )

and compact_hd( VALA( C( v1, v2 ) ) ) = 
    VALA v1
  | compact_hd( CONSA( e1, e2 ) ) = 
    compact( e1 )
  | compact_hd( STATIC( e ) ) = 
    compact_static( STATIC( compact_hd( e ) ) )
  | compact_hd( e ) = 
    HDA( e )

and compact_static( STATIC( STATIC( e ) ) ) = 
    compact_static( STATIC( e ) )
  | compact_static( CONSA( STATIC( e1 ), STATIC( e2 ) ) ) = 
    STATIC( CONSA( e1, e2 ) )
  | compact_static( e ) = 
    e
;

fun substitute( e, VARA ) = e
  | substitute( e, VALA v ) = VALA v
  | substitute( e, CONSA( e1, e2 ) ) = 
    CONSA( substitute( e, e1 ), substitute( e, e2 ) )
  | substitute( e, HDA( e1 ) ) = 
    HDA( substitute( e, e1 ) )
  | substitute( e, TLA( e1 ) ) = 
    TLA( substitute( e, e1 ) )
  | substitute( e, EQA( e1, e2 ) ) =
    EQA( compact( substitute( e, e1 ) ), 
         compact( substitute( e, e2 ) ) )
  | substitute( e, LIFT( e1 ) ) =
    LIFT( substitute( e, e1 ) )
  | substitute( e, STATIC( e1 ) ) =
    STATIC( substitute( e, e1 ) )
;

end
;
