structure Main =
struct

open LPparser;
open Command;

(* Constants used in self interpreter *)

val assign_ = C(N,N);               (* 1 *)
val seq =     C( N, assign_ );      (* 2 *)
val loop =    C( N, seq );          (* 3 *)
val if2 =     C( N, loop );         (* 4 *)
val if1 =     C( N, if2 );          (* 5 *)
val repeat =  C( N, if1 );          (* 6 *)

val var =     C(N,N);               (* 1 *)
val va =      C( N, var );          (* 2 *)
val cons =    C( N, va );           (* 3 *)
val hd =      C( N, cons );         (* 4 *)
val tl =      C( N, hd );           (* 5 *)
val eq =      C( N, tl );           (* 6 *)

fun transP2DExpression( VAR ) = 
    C( var, N )
  | transP2DExpression( VAL v ) = 
    C( va, v )
  | transP2DExpression( CONS( e1, e2 ) ) = 
    C( cons, C( transP2DExpression( e1 ), 
                transP2DExpression( e2 ) ) )
  | transP2DExpression( HD( e ) ) = 
    C( hd, transP2DExpression( e ) )
  | transP2DExpression( TL( e ) ) = 
    C( tl, transP2DExpression( e ) )
  | transP2DExpression( EQ( e1, e2 ) ) = 
    C( eq, C( transP2DExpression( e1 ), transP2DExpression( e2 ) ) )
;

fun transP2DCommand( ASSIGN( e ) ) = 
    C( assign_, transP2DExpression( e ) )
  | transP2DCommand( SEQ( c1, c2 ) ) = 
    C( seq, C( transP2DCommand( c1 ), transP2DCommand( c2 ) ) )
  | transP2DCommand( LOOP( c ) ) = 
    C( loop, transP2DCommand( c ) )
  | transP2DCommand( IF2( e, c1, c2 ) ) = 
    C( if2, C( transP2DExpression( e ), 
               C( transP2DCommand( c1 ), transP2DCommand( c2 ) ) ) )
  | transP2DCommand( IF1( e, c ) ) = 
    C( if1, C( transP2DExpression( e ), transP2DCommand( c ) ) )
  | transP2DCommand( REPEAT ) = 
    C( repeat, N )
;

fun transP2D( p ) = transP2DCommand( p );

fun lp2dUsage() =
    let
        val _ = print( "lp2d <name.lp>\n" );
    in
        1
    end
;

fun lp2dRun( NONE ) =
    let
        val prog_ast = parse();
        val data = transP2D( prog_ast );
        val _ = print( Value.toString( data ) ^ "\n" );
    in
        0
    end
  | lp2dRun( SOME( filename ) ) =
    let
        val prog_ast = parseFile( filename );
        val data = transP2D( prog_ast );
        val _ = print( Value.toString( data ) ^ "\n" );
    in
        0
    end
;

fun main( cmdline, arg ) =
    if length( arg ) = 0
    then lp2dRun( NONE )
    else if length( arg ) <> 1
    then lp2dUsage()
    else lp2dRun( SOME( List.hd( arg ) ) )
;

end
;
