structure Main =
struct

open LPparser;
open ValueParser;
open Interpreter;

fun lpintUsage() =
    let
        val _ = print( "lpint <options> " ^
                       "filename\n" );
        val _ = print( "        expects " ^
                       "initial value on std_in\n" );
        val _ = print( "  -O    optimize " ^
                       "input program before running\n" );
    in
        1
    end
;

fun lpintRun( filename, options ) =
    let
        val prog_ast = parseFile( filename );

        val optimize = List.find (fn x => x = "-O" ) options
                       <> NONE;

        val prog_ast = if optimize 
                       then Tools.fix( Normalization.normalize, 
                                       prog_ast )
                       else prog_ast;
        val prog_ast = if optimize
                       then Optimizer.optimize( prog_ast )
                       else prog_ast;

        val value_ast = parse();

        val result = timed_interpretation( prog_ast, value_ast );
        val _ = print( "result: " ^ 
                       Value.toString( #1( result ) ) ^ "\n" );
        val _ = print( "time: " ^ 
                       Int.toString( #2 result ) ^ "\n" );
    in
        0
    end
;

fun optionfilter( str ) =
    let
        val strlist = explode str;
    in
        hd( strlist ) = #"-"
    end
;

fun main( cmdline, arg ) =
    let
        val options = List.filter optionfilter arg;
        val file = List.filter 
                       ( fn x => not( optionfilter x ) ) arg;
    in
        if length( file ) <> 1
        then lpintUsage()
        else lpintRun( hd( file ), options )
             handle ValueException( msg ) =>
                    let
                        val _ = print( "Value error: " ^ 
                                       msg ^ "\n" );
                    in
                        1
                    end
                  | SyntaxException( msg ) =>
                    let
                        val _ = print( "Syntax error: " ^ 
                                       msg ^ "\n" );
                    in
                        1
                    end
                  | ex =>
                    let
                    val _ = print( "General exception error\n" );
                    in
                        1
                    end
    end
;

end
;
