TESTS:=testreport

TEST_REPORTS:=test_lpint.txt 
TEST_REPORTS+=test_interpreter.txt 
TEST_REPORTS+=test_interpreter2.txt 
TEST_REPORTS+=test_spec.txt 
TEST_REPORTS+=test_specInt.txt
TEST_REPORTS+=test_specInt2.txt

SET1:=$(basename $(notdir $(wildcard tests/Set1/*.lp)))
SET2:=$(basename $(notdir $(wildcard tests/Set2/*.lpa)))

LPINT_PROGS:=$(addprefix tests/lpint/,$(addsuffix .lp,$(SET1))) 
LPINT_PROGS+=$(addprefix tests/lpint/,$(addsuffix .lp, $(SET2)))
LPINT_INPUT:=$(addsuffix .input,$(basename $(LPINT_PROGS)))
LPINT_EXPECTED:=$(addsuffix .expected,$(basename $(LPINT_PROGS)))

INTERPRETER:=tests/interpreter/interpreter.lp
INTERPRETER_INPUT:=$(addprefix tests/interpreter/,$(addsuffix .input,$(SET1))) 
INTERPRETER_INPUT+=$(addprefix tests/interpreter/,$(addsuffix .input, $(SET2)))
INTERPRETER_EXPECTED:=$(addsuffix .expected,$(basename $(INTERPRETER_INPUT)))

INTERPRETER2:=tests/interpreter2/interpreter.lp
INTERPRETER2_INPUT:=$(addprefix tests/interpreter2/,$(addsuffix .input,$(SET1))) 
INTERPRETER2_INPUT+=$(addprefix tests/interpreter2/,$(addsuffix .input, $(SET2)))
INTERPRETER2_EXPECTED:=$(addsuffix .expected,$(basename $(INTERPRETER2_INPUT)))

SPEC_PROGS:=$(addprefix tests/spec/,$(addsuffix .lp, $(SET2)))
SPEC_INPUT:=$(addsuffix .input,$(basename $(SPEC_PROGS)))
SPEC_EXPECTED:=$(addsuffix .expected,$(basename $(SPEC_PROGS)))

SPEC_INTERPRETER:=tests/specInt/interpreter.lpa
SPEC_INT_PROGS:=$(addprefix tests/specInt/,$(addsuffix .lp,$(SET1))) 
SPEC_INT_PROGS+=$(addprefix tests/specInt/,$(addsuffix .lp, $(SET2)))
SPEC_INT_INPUT:=$(addsuffix .input,$(basename $(SPEC_INT_PROGS)))
SPEC_INT_EXPECTED:=$(addsuffix .expected,$(basename $(SPEC_INT_PROGS)))

SPEC_INTERPRETER2:=tests/specInt2/interpreter.lp
SPEC_INT2_INPUT:=$(addprefix tests/specInt2/,$(addsuffix .input,$(SET1))) 
SPEC_INT2_INPUT+=$(addprefix tests/specInt2/,$(addsuffix .input, $(SET2)))
SPEC_INT2_EXPECTED:=$(addsuffix .expected,$(basename $(SPEC_INT2_INPUT)))

testreport: $(TEST_REPORTS)

test_lpint.txt: $(BIN_DIR)/tester.sh $(LPINT) $(LPINT_PROGS) $(LPINT_INPUT) $(LPINT_EXPECTED)
	@echo "testing lpint"
	@for i in $(basename $(LPINT_PROGS)); do ./bin/tester.sh $$i $$i.lp $$i.input $$i.expected; done

test_interpreter.txt: $(BIN_DIR)/tester.sh $(LPINT) $(INTERPRETER_INPUT) $(INTERPRETER_EXPECTED) $(INTERPRETER)
	@echo "Testing interpreter"
	@for i in $(basename $(INTERPRETER_INPUT)); do ./bin/tester.sh $$i $(INTERPRETER) $$i.input $$i.expected -O; done

test_interpreter2.txt: $(BIN_DIR)/tester.sh $(LPINT) $(INTERPRETER2_INPUT) $(INTERPRETER2_EXPECTED) $(INTERPRETER2)
	@echo "Testing interpreter on interpreter"
	@for i in $(basename $(INTERPRETER2_INPUT)); do ./bin/tester.sh $$i $(INTERPRETER2) $$i.input $$i.expected -O; done

test_spec.txt: $(BIN_DIR)/tester.sh $(LPINT) $(SPEC_PROGS) $(SPEC_INPUT) $(SPEC_EXPECTED)
	@echo "Testing specializer"
	@for i in $(basename $(SPEC_PROGS)); do ./bin/tester.sh $$i $$i.lp $$i.input $$i.expected; done

test_specInt.txt: $(BIN_DIR)/tester.sh $(LPINT) $(SPEC_INT_PROGS) $(SPEC_INT_INPUT) $(SPEC_INT_EXPECTED)
	@echo "Testing specializer interpreter"
	@for i in $(basename $(SPEC_INT_PROGS)); do ./bin/tester.sh $$i $$i.lp $$i.input $$i.expected; done

test_specInt2.txt: $(BIN_DIR)/tester.sh $(LPINT) $(SPEC_INTERPRETER2) $(SPEC_INT2_INPUT) $(SPEC_INT2_EXPECTED)
	@echo "Testing specialize interpreter with interpreter"
	@for i in $(basename $(SPEC_INT2_INPUT)); do ./bin/tester.sh $$i $(SPEC_INTERPRETER2) $$i.input $$i.expected; done

########################################
# Pattern rules for test/lpint
########################################

tests/lpint/%.lp: tests/Set1/%.lp tests/module.mk
	@cp $< $@

tests/lpint/%.lp: tests/Set2/%.lpa $(LPA2LP) tests/module.mk
	@$(BIN_DIR)/lpa2lp $< > $@

tests/lpint/%.input: tests/Set1/%.input tests/module.mk
	@cp $< $@

tests/lpint/%.input: tests/Set2/%.lps tests/Set2/%.lpd tests/module.mk
	@rm -f $@
	@echo "(" >> $@
	@cat tests/Set2/$*.lps >> $@
	@echo "," >> $@
	@cat tests/Set2/$*.lpd >> $@
	@echo ")" >> $@

tests/lpint/%.expected: tests/Set1/%.expected tests/module.mk
	@cp $< $@

tests/lpint/%.expected: tests/Set2/%.expected tests/module.mk
	@cp $< $@

########################################
# Pattern rules for test/interpreter
########################################

$(INTERPRETER): lpa-progs/interpreter.lpa $(LPA2LP) tests/module.mk
	@$(BIN_DIR)/lpa2lp $< > $@

tests/interpreter/%.input: tests/Set1/%.lp tests/Set1/%.input $(LP2D) tests/module.mk
	@rm -f $@
	@echo "(" >> $@
	@$(BIN_DIR)/lp2d tests/Set1/$*.lp >> $@
	@echo "," >> $@
	@cat tests/Set1/$*.input >> $@
	@echo ")" >> $@

tests/interpreter/%.input: tests/Set2/%.lps tests/Set2/%.lpd $(LPA2LP) $(LP2D) tests/module.mk
	@rm -f $@
	@echo "(" >> $@
	@$(BIN_DIR)/lpa2lp tests/Set2/$*.lpa | $(BIN_DIR)/lp2d >> $@
	@echo ",(" >> $@
	@cat tests/Set2/$*.lps >> $@
	@echo "," >> $@
	@cat tests/Set2/$*.lpd >> $@
	@echo "))" >> $@

tests/interpreter/%.expected: tests/Set1/%.expected tests/module.mk
	@cp $< $@

tests/interpreter/%.expected: tests/Set2/%.expected tests/module.mk
	@cp $< $@

########################################
# Pattern rules for test/interpreter2
########################################

$(INTERPRETER2): lpa-progs/interpreter.lpa $(LPA2LP) tests/module.mk
	@$(BIN_DIR)/lpa2lp $< > $@

tests/interpreter2/%.input: tests/Set1/%.lp tests/Set1/%.input $(INTERPRETER2) $(LP2D) tests/module.mk
	@rm -f $@
	@echo "(" >> $@
	@$(BIN_DIR)/lp2d $(INTERPRETER2) >> $@
	@echo ",(" >> $@
	@$(BIN_DIR)/lp2d tests/Set1/$*.lp >> $@
	@echo "," >> $@
	@cat tests/Set1/$*.input >> $@
	@echo "))" >> $@

tests/interpreter2/%.input: tests/Set2/%.lps tests/Set2/%.lpd $(INTERPRETER2) $(LPA2LP) $(LP2D) tests/module.mk
	@rm -f $@
	@echo "(" >> $@
	@$(BIN_DIR)/lp2d $(INTERPRETER2) >> $@ 
	@echo ",(" >> $@
	@$(BIN_DIR)/lpa2lp tests/Set2/$*.lpa | $(BIN_DIR)/lp2d >> $@
	@echo ",(" >> $@
	@cat tests/Set2/$*.lps >> $@
	@echo "," >> $@
	@cat tests/Set2/$*.lpd >> $@
	@echo ")))" >> $@

tests/interpreter2/%.expected: tests/Set1/%.expected
	@cp $< $@

tests/interpreter2/%.expected: tests/Set2/%.expected
	@cp $< $@

########################################
# Pattern rules for tests/spec
########################################

tests/spec/%.lp: tests/Set2/%.lpa tests/Set2/%.lps $(SPEC)
	@$(BIN_DIR)/spec tests/Set2/$*.lpa $@ < tests/Set2/$*.lps

tests/spec/%.input: tests/Set2/%.lpd
	@cp $< $@

tests/spec/%.expected: tests/Set2/%.expected
	@cp $< $@

########################################
# Pattern rules for tests/specInt
########################################

$(SPEC_INTERPRETER): lpa-progs/interpreter.lpa tests/module.mk
	@cp $< $@

tests/specInt/%.lp: tests/Set1/%.lp $(LP2D) $(SPEC) $(SPEC_INTERPRETER) tests/module.mk
	@$(BIN_DIR)/lp2d $< | $(BIN_DIR)/spec $(SPEC_INTERPRETER) $@

tests/specInt/%.lp: tests/Set2/%.lpa $(LPA2LP) $(SPEC) $(SPEC_INTERPRETER) tests/module.mk
	@$(BIN_DIR)/lpa2lp $< | $(BIN_DIR)/lp2d | $(BIN_DIR)/spec $(SPEC_INTERPRETER) $@

tests/specInt/%.input: tests/Set1/%.input tests/module.mk
	@cp $< $@

tests/specInt/%.input: tests/Set2/%.lps tests/Set2/%.lpd tests/module.mk
	@rm -f $@
	@echo "(" >> $@
	@cat tests/Set2/$*.lps >> $@
	@echo "," >> $@
	@cat tests/Set2/$*.lpd >> $@
	@echo ")" >> $@

tests/specInt/%.expected: tests/Set1/%.expected tests/module.mk
	@cp $< $@

tests/specInt/%.expected: tests/Set2/%.expected tests/module.mk
	@cp $< $@

########################################
# Pattern rules for tests/specInt2
########################################

$(SPEC_INTERPRETER2): lpa-progs/interpreter.lpa $(LPA2LP) $(LP2D) $(SPEC) tests/module.mk
	@$(BIN_DIR)/lpa2lp $< | $(BIN_DIR)/lp2d | $(BIN_DIR)/spec $< $@

tests/specInt2/%.input: tests/Set1/%.lp tests/Set1/%.input $(LP2D) tests/module.mk
	@rm -f $@
	@echo "(" >> $@
	@$(BIN_DIR)/lp2d tests/Set1/$*.lp >> $@
	@echo "," >> $@
	@cat tests/Set1/$*.input >> $@
	@echo ")" >> $@

tests/specInt2/%.input: tests/Set2/%.lps tests/Set2/%.lpd $(LPA2LP) $(LP2D) tests/module.mk
	@rm -f $@
	@echo "(" >> $@
	@$(BIN_DIR)/lpa2lp tests/Set2/$*.lpa | $(BIN_DIR)/lp2d >> $@
	@echo ",(" >> $@
	@cat tests/Set2/$*.lps >> $@
	@echo "," >> $@
	@cat tests/Set2/$*.lpd >> $@
	@echo "))" >> $@

tests/specInt2/%.expected: tests/Set1/%.expected tests/module.mk
	@cp $< $@

tests/specInt2/%.expected: tests/Set2/%.expected tests/module.mk
	@cp $< $@

########################################
# Clean files
########################################

CLEAN_TEST_FILES:=$(LPINT_PROGS)
CLEAN_TEST_FILES+=$(LPINT_INPUT)
CLEAN_TEST_FILES+=$(LPINT_EXPECTED)

CLEAN_TEST_FILES+=$(INTERPRETER)
CLEAN_TEST_FILES+=$(INTERPRETER_INPUT)
CLEAN_TEST_FILES+=$(INTERPRETER_EXPECTED)

CLEAN_TEST_FILES+=$(INTERPRETER2)
CLEAN_TEST_FILES+=$(INTERPRETER2_INPUT)
CLEAN_TEST_FILES+=$(INTERPRETER2_EXPECTED)

CLEAN_TEST_FILES+=$(SPEC_PROGS)
CLEAN_TEST_FILES+=$(SPEC_INPUT)
CLEAN_TEST_FILES+=$(SPEC_EXPECTED)

CLEAN_TEST_FILES+=$(SPEC_INTERPRETER)
CLEAN_TEST_FILES+=$(SPEC_INT_PROGS)
CLEAN_TEST_FILES+=$(SPEC_INT_INPUT)
CLEAN_TEST_FILES+=$(SPEC_INT_EXPECTED)

CLEAN_TEST_FILES+=$(SPEC_INTERPRETER2)
CLEAN_TEST_FILES+=$(SPEC_INT2_INPUT)
CLEAN_TEST_FILES+=$(SPEC_INT2_EXPECTED)

CLEAN_FILES+=$(CLEAN_TEST_FILES)

clean_test:
	@rm -fr $(CLEAN_TEST_FILES)

.PHONY: test_report clean_test $(TEST_REPORTS)